package kpalin.alabra;
import java.awt.*;
import java.io.File;

/** Here we create a window for application and prepare data for the game.
    This is the highest class specificly for application so here we have to
    handle file management and menu stuf.

    @author Kimmo Palin
*/

public class AppForm extends Frame{
  /** Menubar for the window. Just in case taken here if we hapen to need it
      sometimes somewhere.
  */
  private ChessAppMB frameMenu;


  /** This is the real gamething. From this on, applet and application are
      all alike.
  */
  private ChessPlay chessGame;

  /** An object controlling saving and loading of the game. */
  private FileIO saveGame;

  /** Loads a saved file. */
  public void loadGame() {
    chessGame.pause();

    if(saveGame.loadFile()) {
      //setVisible(false);
      if(chessGame!=null)
	remove(chessGame);
      chessGame=(ChessPlay)saveGame.getObject();
      add(chessGame);
      //setVisible(true);
      validate();
    } else
      System.err.println("Can't load file!");

    chessGame.unpause();
  }

  /** Saves this game with old name or asks a new one if this is a first save.
    */
  public void saveGame() {
    chessGame.pause();
    if(!saveGame.saveFile()) {
      System.err.println("Can't save file!");
    }
    chessGame.unpause();
  }
  

  /** Asks user a name of a file where to save this game.
    */
  public void saveGameAs() {
    chessGame.pause();
    if(!saveGame.saveFileAs()) {
      System.err.println("Can't save file!");
    }
    chessGame.unpause();
  }
  


  /** Creates new application window.
   @param fname Name of the file to be loaded.
   @param gifdir Name of the directory where we can find the gifs for pieces.
   @param tileWidth Width (and height) of a tile on the chessboard.
  */ 
  AppForm(String fname,String gifdir,ChessSettings set) {
    super(CommonStaticData.appName+" "+CommonStaticData.version);

    set.initStanding();

    addWindowListener(new AppWinCtrl());
    readImages(gifdir);

    if(Piece.getIcon()!=null)
      setIconImage(Piece.getIcon());

    saveGame=new FileIO(this);

    if(fname!=null && fname.length()>0) {
      saveGame.setFileName(fname);
      if(saveGame.loadData()) {
	chessGame=(ChessPlay)saveGame.getObject();
	chessGame.unpause();
      }
    }

    if(chessGame==null) {
      chessGame=new ChessPlay(set);
      saveGame.setObject(chessGame);
    }

    frameMenu=new ChessAppMB(this);
    setMenuBar(frameMenu);
    add(chessGame);

    pack();
    setVisible(true);
  }

  /** Reseting the game
   */
  void resetGame() {
    chessGame.resetGame();
  }
  /** Pause if iconified.
      @see ChessPlay
      @see CtrlPanel
  */
  void pause() { chessGame.pause(); }

  /** Unpause if de-iconified.
      @see ChessPlay
      @see CtrlPanel
  */
  void unpause() { chessGame.unpause(); }

  /** This is only method that can kill the application
      @param code Exit code.
   */
  void exitApp(int code) {
    setVisible(false);
    System.exit(code);
  }

  /** Here we start reading the Images. This code is specific to application
      and can't be shared with applet.
      @param basedir Directory where the images should be found.
  */
  void readImages(String basedir) {
    Toolkit tk=getToolkit();
    File imgFile;

    if(basedir==null || basedir.length()==0)
      basedir="";
    else
      basedir=basedir+File.separator;


    for(int side=0;side<Piece.IMAGENAME.length;side++) {
      for(int type=0;type<Piece.IMAGENAME[side].length;type++) {

	//For some braindead reason, getImage always returns valid
	//Image object so we have to check existance of images ourselves.
	imgFile=new File(basedir+Piece.IMAGENAME[side][type]);

	if(imgFile.canRead()==false) {
	  System.err.println("Can't find required images from directory "+basedir);
	  System.err.println("Please, copy the images to correct directory and run again.");
	  System.exit(1);
	}
       	Piece.addImage(side,type,tk.getImage(imgFile.toString()));
      }
    }
  }
}


