package kpalin.alabra;
import java.awt.*;

/**  This is the visual representation of a tile on the board.
 @author Kimmo Palin
*/

public class BoardTile extends Canvas {
  /** Tile color for white tiles.
   */
  static final Color WHITETILE=Color.white;

  /** Tile color for black tiles.
   */
  static final Color BLACKTILE=Color.green;

  /** Tile color for selected tile.
   */
  static final Color SELECTEDTILE=Color.gray;


  /** Piece on this tile.
  */
  private Piece piece;

  /** Wheather this tile is selected.
   */
  private boolean selected;

  /** Position of the tile. Read only.
   */
  final int xpos;
  final int ypos;

  /** Creates new empty Tile.
      @param x Column of the tile.
      @param y Row of the tile.
  */ 
  BoardTile(int x,int y) {
    Dimension d=ChessSettings.getTileDim();

    xpos=x;
    ypos=y;

    /* This is deprecated api but say that to netscape */
    //resize(d.width,d.height);
    setSize(d);

    setBgColor();
  }

  /** Accessor for piece on this tile.
      @return Piece on this tile. null if there isn't one.
  */
   Piece getPiece() { return piece; }

  /** Accessor for piece on this tile.
      @param p Piece to set on this tile. Null to clear tile.
  */
   void setPiece(Piece p) { piece=p; repaint(); }

  /** Sets normal background color for a tile.
   */
  private void setBgColor() {
    if((xpos+ypos)%2==0)
      setBackground(BLACKTILE);
    else
      setBackground(WHITETILE);
  }

  /** Here we draw the image of the piece on the tile if there is one.
   */
  public  void paint(Graphics g) {
    if(piece!=null) {
      Dimension d=getSize();
      g.drawImage(piece.getImage(),0,0,d.width,d.height,this);
    }
  }
  /** Switch for selection. Changes background color. 
   */
   void setSelected() { setSelected(!selected); }

  /** Changes selection status.
      @param select True if select, false if de-select.
  */
   void setSelected(boolean select) {
    selected=select;
    if(select)
      setBackground(SELECTEDTILE);
    else
      setBgColor();

    repaint();
  }

  /** Helps with move-history keeping.
      @returns Column code (a-h) of the tile.
  */
  char getColumnCode() { return BoardGrid.IDSTRS[xpos]; }

  public String toString() {
    return ""+BoardGrid.IDSTRS[xpos]+(1+ypos);
  }
}




