package kpalin.alabra;
import java.awt.*;
import java.text.*;
import java.io.*;
import java.util.*;

/** This controls the timers.

    @author Kimmo Palin
*/

public class ChessTimer extends Panel implements Runnable,Externalizable {
  /** Side of the player. White or Black.
   */
  private Label side;

  /** The time shown on the panel.
   */
  private Label timeLabel;

  /** Integer representation of the time.
   */
  private int timeSec=0;


  /** Starting time of the timers. If this is >0, the timers
      run downwards, else upwards.
  */
  private int timeBase=0;

  /** The counters are added this value every second.
   */
  private int timeDelta=1;

  /** Thread controling the clock.
   */
  private Thread th;

  /** Signaling interface towards Observer.
   */
  private ClockListen timeElapsed;

  /** This avoids fetching NumberFormat information every second.
   */
  private final NumberFormat nf=NumberFormat.getInstance();


  /** This is useless constructor for programmer but Externalizable-interface
      uses this (public no-arg constructor).
      */
  public ChessTimer() {
    setLayout(new GridLayout(1,2));
    nf.setMinimumIntegerDigits(2);
  }

  /** Creates a new chess timer for specific player.
      @param who Name whose timer this is.
      @param sideCode Number id of the player, for callback.
      @param timeLimit Size of the timeshare.
      @param obs Observer for the callback.
   */
  ChessTimer(String who,int sideCode,int timeLimit,Observer obs) {
    this();

    setTimeBase(timeLimit);
    timeSec=timeBase;

    adjustTimeDelta();

    timeElapsed=new ClockListen(sideCode,obs);

    side=new Label(who,Label.CENTER);
    timeLabel=new Label(this.toString(),Label.CENTER);

    add(side);
    add(timeLabel);
  }

  /** Sets new time base.
      @param base New time base.
   */
  private void setTimeBase(int base) {
    if(base>0)
      timeBase=base;
    else
      timeBase=0;
  }
  /** Gives a new timeslize for this timer.
   */
  public void newTimeSlize() {
    timeSec+=timeBase;
  }
  /** Sets the timeDelta-attribute to correct direction.
   */
  private void adjustTimeDelta() {
    if(timeBase>0)
      timeDelta=-1;
    else
      timeDelta=1;
  }

  /** Resets the timer to stop at timeBase
   */
  public void reset() {
    if(th!=null)
      suspend();
    th=null;
    timeSec=timeBase;
    timeLabel.setText(this.toString());
  }

  /** This controls the timers.
   */
  public void run() {
    while(true) {
      try {
	th.sleep(1000);
	timeSec+=timeDelta;  //add one second to timer.
	synchronized(this) {
	  timeLabel.setText(this.toString());
	}
	if(timeSec<1) {
	  timeElapsed.allert();
	}
      } catch(InterruptedException e) { }
    }
  }

  /** Time formater for Label. Synchronized because of nf-instance.
   */
  public synchronized String toString() {
    return nf.format(timeSec/60)+":"+nf.format(timeSec%60);
  }

  /** Resumes the time counting.
   */
  public synchronized void resume() {
    try {
      th.resume();
    } catch(NullPointerException e) {
      th=new Thread(this);
      th.start();
    } catch(Exception e) {
      System.out.println(e);
    }
  }
  /** Suspends the counter for now.
   */
  public synchronized void suspend() {
    try {
      th.suspend();
    }catch(NullPointerException e) {}
    catch(Exception e) { }  //netscape.security.AppletSecurityException
  }

  /** Checks if we are testing from current thread. (Needed for the alarm.)
   */
  public boolean isCurrentThread() {
    return th==Thread.currentThread();
  }

  /** Stops the counter permanently.
   */
  public synchronized void stop() {
    try {
      th.stop();  //Time allert kill jams here
    } catch(Exception e) { }
  }

  /** Returns elapsed time in seconds.
    */
  public int getTime() { return timeSec; }


  /** Used for saving this object. This is a must, because it's impossible
      to save Thread-object.
  */
  public void writeExternal(ObjectOutput out) throws IOException {
    out.writeObject(side);
    out.writeObject(timeLabel);
    out.write(timeSec);
    out.write(timeBase);
    out.write(timeDelta);
    out.writeObject(timeElapsed);
  }

  /** Used for reading saved ChessTimer. Doesn't read Thread because it's
      not saved.
  */
  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
    side=(Label)in.readObject();
    timeLabel=(Label)in.readObject();
    timeSec=in.read();
    timeBase=in.read();
    timeDelta=in.read();
    timeElapsed=(ClockListen)in.readObject();

    add(side);
    add(timeLabel);
  }

}









