package kpalin.alabra;
import java.io.*;
import java.awt.*;

/** Here we control all saving and loading of game. The game is saved
    brutaly by saving the ChessPlay object, but this class is writen
    for general object so we can change the implementation easily.

    @author Kimmo Palin
*/

public class FileIO extends FileDialog implements FilenameFilter {

  /** Identification suffix of the files.
    */
  private final String fileSuffix=".chs";

  private final String SAVETITLE="Save game";
  private final String LOADTITLE="Load game";

  /** Object that this object controls. */
  private Object saveObject=null;

  /** Name of the file where current game has been saved.
    */
  private String savefile=null;

  /** Creates new object for handling game saving and loading.
    @param parent Parent frame of the possible FileDialog
    */
  FileIO(Frame parent) {
    super(parent);
    setFilenameFilter(this);
  }

  /** Creates new object for handling game saving and loading.
    @param parent Parent frame of the possible FileDialog
    @param save The object to be saved.
  */
  FileIO(Frame parent, Object save) {
    this(parent);
    setObject(save);
  }

  /** Sets new object for this FileIO to handle.
    */
  public void setObject(Object target) {
    saveObject=target;
  }

  public void setFileName(String name) {
    savefile=name;
  }
  public String getFileName() {
    return savefile;
  }
  /** Returns the target of this object. 
    */
  public Object getObject() {
    return saveObject;
  }

  /** Controls what files to show on dialog and what not.
    */
  public boolean accept(File dir, String name) {
    return name.endsWith(fileSuffix);
  }


  public boolean saveFileAs() {
    getSaveName();

    return saveData();
  }

  public boolean saveFile() {
    if(savefile==null)
      getSaveName();
    return saveData();
  }

  private void getSaveName() {
    setMode(SAVE);
    setTitle(SAVETITLE);

    if(savefile!=null)
      setFile(savefile);
    show();

    if(getFile()!=null)
      savefile=getDirectory()+getFile();
  }

  /** Saves game with a given name.
      @return true if save was succesful, otherwise false.
  */
  public boolean saveData() {
    if(savefile==null)
      return false;

    try {
      ObjectOutputStream strm=new ObjectOutputStream(new FileOutputStream(savefile));
    
      strm.writeObject(saveObject);
      strm.close();
    }catch(IOException e) { return false; }
    catch(Exception e) {
      System.err.println("Terrible error in saveFile(): "+e);
      return false;
    }

    return true;
  }

  /** Asks the name of the game to be load and loads it.
    */
  public boolean loadFile() {
    getLoadName();
    return loadData();
  }

  /** Asks the name of the game to be load.
    */
  private void getLoadName() {
    setMode(LOAD);
    setTitle(LOADTITLE);

    if(savefile!=null)
      setFile(savefile);
    show();

    if(getFile()!=null)
      savefile=getDirectory()+getFile();
  }

  /** Loads the object from the savefile.
    */
  public boolean loadData() {
    if(savefile==null)
      return false;

    Object loaded=null;

    try {
      ObjectInputStream strm=new ObjectInputStream(new FileInputStream(savefile));
    
      loaded=strm.readObject();
      strm.close();
    }catch(StreamCorruptedException e) {
      System.err.println("File format error! This is not Jchess savegame!");
      savefile=null;
    }catch(Exception e) { 
      System.err.println("Error loading file!"+e);
      return false; 
    }

    if(loaded==null)
      return false;

    saveObject=loaded;

    return true;

  }
}
