package kpalin.alabra;
import java.applet.*;
import java.net.*;
import java.awt.Dimension;

/* This is the starting class of both application and applet. This class 
   reads HTML-params, and parses commandline. For applet, it also
   initializes ChessSettings and starts the game.

    @author Kimmo Palin
*/

public class Jchess extends Applet {
  /** The actual game stuff.
   */
  private ChessPlay chessGame;

  /** Constant for name of GifURL parameter.
   */
  private static String GIFURLPARAM="GifURL";

  /** Constant for name of TileSize parameter.
   */
  private static String TILESIZEPARAM="TileSize";

  /** Constant for name of FontSize parameter.
   */
  private static String FONTSIZEPARAM="FontSize";
  /** Use applied Quickchess rules. All moves in 5 minutes.
   */
  private static String QUICKCHESSPARAM="QuickChess";
  /** Number of moves that must be made in timeperiod.
   */
  private static String MOVESINPERIODPARAM="Moves";
  /** Length of the timeperiod.
   */
  private static String TIMELIMITPARAM="TimeLimit";

  /** Constant for name of fontsize commandline argument.
   */
  private static String FONTSIZEARG="-fontsize";

  /** Constant for name of gifdir commandline argument.
   */
  private static String GIFDIRARG="-gifdir";

  /** Constant for name of TileSize commandline argument.
   */
  private static String TILESIZEARG="-tilesize";

  /** Use applied Quickchess rules. All moves in 5 minutes.
   */
  private static String QUICKCHESSARG="-quickchess";
  /** Number of moves that must be made in timeperiod.
   */
  private static String MOVESINPERIODARG="-moves";
  /** Length of the timeperiod.
   */
  private static String TIMELIMITARG="-timelimit";

  /**
     Application takes one command line parameter: name of the game file to be
     loaded.
  */
  public static void main(String [] args) {
    String loadFileName="";
    int tileWidthHeight=32;
    String gifDirName="";
    int fontSize=24;
    ChessSettings set=new ChessSettings();


    try {
      for(int i=0;i<args.length;i++) {
	if(args[i].compareTo(GIFDIRARG)==0) {
	  gifDirName=args[++i];
	} else if(args[i].compareTo(TILESIZEARG)==0) {
	  try {
	    tileWidthHeight=Integer.parseInt(args[++i]);
	    set.setTileDim(new Dimension(tileWidthHeight,tileWidthHeight));
	  } catch(NumberFormatException e) { i--; }
	} else if(args[i].compareTo(FONTSIZEARG)==0) {
	  try {
	    fontSize=Integer.parseInt(args[++i]);
	    set.setFontSize(fontSize);
	  } catch(NumberFormatException e) { i--; } 
	} else if(args[i].compareTo(QUICKCHESSARG)==0) 
	  set.setQuickChess();
	else if(args[i].compareTo(MOVESINPERIODARG)==0) {
	  try {
	    set.setMoveLimit(Integer.parseInt(args[++i]));
	  } catch(NumberFormatException e) { i--; }
	} else if(args[i].compareTo(TIMELIMITARG)==0) {
	  try {
	    set.setTimeLimit(Integer.parseInt(args[++i]));
	  } catch(NumberFormatException e) { i--; }
	} else
	  loadFileName=args[i];
      }
    } catch(ArrayIndexOutOfBoundsException e) {}

    new AppForm(loadFileName,gifDirName,set);
  }


  /** This stops the game when applet is taken of the screen.
    */
  public void stop() {
    chessGame.pause();
  }

  /** Unpauses the game when we come back to view.
    */
  public void start() {
    chessGame.unpause();
  }
    

  /** Run at applet startup.
   */
  public void init() {
    int tileSize,fontSize;
    readImages();


    ChessSettings set=new ChessSettings();
    set.initStanding();

    try {
      tileSize=Integer.parseInt(getParameter(TILESIZEPARAM));
      set.setTileDim(new Dimension(tileSize,tileSize));
    } catch(Exception e) {
      ChessSettings.setTileDim(new Dimension(32,32));
    }
    try {
      fontSize=Integer.parseInt(getParameter(FONTSIZEPARAM));
      set.setFontSize(fontSize);
    } catch(Exception e) {} 
    try {
      set.setTimeLimit(Integer.parseInt(getParameter(TIMELIMITPARAM)));
    } catch(Exception e) {} 
    try {
      set.setTimeLimit(Integer.parseInt(getParameter(TIMELIMITPARAM)));
    } catch(Exception e) {} 
    try {
      set.setMoveLimit(Integer.parseInt(getParameter(MOVESINPERIODPARAM)));
    } catch(Exception e) {} 
    try {
      if(Integer.parseInt(getParameter(QUICKCHESSPARAM))!=0) 
	set.setQuickChess();
    } catch(Exception e) {} 


    chessGame=new ChessPlay(set);
    add(chessGame);
  }
  /** Here we start reading the Images. This code is specific to applet
      and can't be shared with application.
  */
  void readImages() {
    URL base;

    try {
      base=new URL(getCodeBase(),getParameter(GIFURLPARAM));
    } catch(MalformedURLException e) {
      base=getCodeBase();
    }

    for(int side=0;side<Piece.IMAGENAME.length;side++) {
      for(int type=0;type<Piece.IMAGENAME[side].length;type++) {
	Piece.addImage(side,type,getImage(base,Piece.IMAGENAME[side][type]));
      }
    }
  }

}
