package kpalin.alabra;
import java.util.*;

/** This is just a datatransfer class co-operating with PawnPromote
    and ChessPlay. The Object returned from this Observable is the
    Piece to which the player wants to promote the pawn.

    @see Observable
    @author Kimmo Palin
*/

public class PromotionListen extends Observable {
  public final BoardTile FROM;
  public final BoardTile TO;

  private Piece promoteTo;

  private boolean haveData=false;

  /** Returns the Piece in this object.
   */
  public synchronized Piece getPiece() {
    clearChanged();
    return promoteTo;
  }

  /** Sets selected Piece to this object and notifies observers.
   */
  public synchronized void setPiece(Piece newPiece) {
    promoteTo=newPiece;
    setChanged();
    notifyObservers(promoteTo);
  }

  public boolean hasChanged() {
    return haveData;
  }
  protected synchronized void clearChanged() {
    haveData=false;
  }
  protected synchronized void setChanged() {
    haveData=true;
  }

  /** Creates new Observable PromotionListen.
      @param from Starting tile a the move.
      @param to Target tile a the move.
  */
  PromotionListen(BoardTile from, BoardTile to) {
    FROM=from;
    TO=to;
  }
}
