package kpalin.alabra;
import java.awt.*;
import java.text.*;

/** This controls the timers.

    @author Kimmo Palin
*/

public class ChessTimer extends Panel implements Runnable {
  /** Side of the player. White or Black.
   */
  private final Label side;

  /** The time shown on the panel.
   */
  private Label timeLabel;

  /** Integer representation of the time.
   */
  private int timeSec;

  /** Thread controling the clock.
   */
  private Thread th;

  /** This avoids fetching NumberFormat information every second.
   */
  private final NumberFormat nf=NumberFormat.getInstance();

  /** Creates a new chess timer for specific player.
   */
  ChessTimer(String who) {
    timeSec=0;
    setLayout(new GridLayout(1,2));
    nf.setMinimumIntegerDigits(2);

    side=new Label(who,Label.CENTER);
    timeLabel=new Label(this.toString(),Label.CENTER);

    add(side);
    add(timeLabel);
  }
  /** Resets the timer to stop at 00:00
   */
  public void reset() {
    if(th!=null)
      th.suspend();
    timeSec=0;
    timeLabel.setText(this.toString());
  }

  /** This controls the timers.
   */
  public void run() {
    while(true) {
      try {
	th.sleep(1000);
	timeSec++;  //add one second to timer.
	timeLabel.setText(this.toString());
      } catch(InterruptedException e) { }
    }
  }

  /** Time formater for Label.
   */
  public String toString() {
    return nf.format(timeSec/60)+":"+nf.format(timeSec%60);
  }
  /** Resumes the time counting.
   */
  void resume() {
    if(th==null) {
      th=new Thread(this);
      th.start();
    } else
      th.resume();
  }
  /** Suspends the counter for now.
   */
  void suspend() {
    if(th!=null)
      th.suspend();
  }
  /** Stops the counter permanently.
   */
  void stop() {
    th.stop();
    th=null;
  }
}

