package kpalin.alabra;
import java.applet.*;
import java.net.*;
import java.awt.Dimension;

/* This is the starting class of both application and applet. This class 
   reads HTML-params, and parses commandline. For applet, it also
   initializes ChessSettings and starts the game.

    @author Kimmo Palin
*/

public class Jchess extends Applet {
  /** The actual game stuff.
   */
  private ChessPlay chessGame;

  /** Constant for name of GifURL parameter.
   */
  private static String GIFURLPARAM="GifURL";

  /** Constant for name of TileSize parameter.
   */
  private static String TILESIZEPARAM="TileSize";

  /** Constant for name of gifdir commandline argument.
   */
  private static String GIFDIRARG="-gifdir";

  /** Constant for name of TileSize commandline argument.
   */
  private static String TILESIZEARG="-tilesize";

  /**
     Application takes one command line parameter: name of the game file to be
     loaded.
  */
  public static void main(String [] args) {
    String loadFileName="";
    int tileWidthHeight=32;
    String gifDirName="";

    try {
      for(int i=0;i<args.length;i++) {
	if(args[i].compareTo(GIFDIRARG)==0) {
	  gifDirName=args[++i];
	} else if(args[i].compareTo(TILESIZEARG)==0) {
	  try {
	    tileWidthHeight=Integer.parseInt(args[++i]);
	  } catch(NumberFormatException e) {}   
	} else
	  loadFileName=args[i];
      }
    } catch(ArrayIndexOutOfBoundsException e) {}

    new AppForm(loadFileName,gifDirName,tileWidthHeight);
  }


  /** This unpauses the game when applet is started or taken back
      to the screen.
  */
  public void start() {
    //chessGame.unpause();
    System.err.println("Start() called!");
  }

  /** This pauses the game when applet is taken of the screen.
    */
  public void stop() {
    chessGame.killAll();
    System.err.println("Stop() called!");
  }

  /** Run at applet startup.
   */
  public void init() {
    int tileSize;
    readImages();


    ChessSettings set=new ChessSettings();
    set.initStanding();

    try {
      tileSize=Integer.parseInt(getParameter("TileSize"));
      ChessSettings.setTileDim(new Dimension(tileSize,tileSize));
    } catch(Exception e) {
      ChessSettings.setTileDim(new Dimension(32,32));
    }
    chessGame=new ChessPlay(set);
    add(chessGame);
  }
  /** Here we start reading the Images. This code is specific to applet
      and can't be shared with application.
  */
  void readImages() {
    URL base;

    try {
      base=new URL(getCodeBase(),getParameter(GIFURLPARAM));
    } catch(MalformedURLException e) {
      base=getCodeBase();
    }

    for(int side=0;side<Piece.IMAGENAME.length;side++) {
      for(int type=0;type<Piece.IMAGENAME[side].length;type++) {
	Piece.addImage(side,type,getImage(base,Piece.IMAGENAME[side][type]));
      }
    }
  }

}
